# @markup text
local: true
state_top: top.sls
root_dir: <%= config[:root_path] %>

file_roots:
 <%= config[:salt_env] %>:
   - <%= os_join(config[:root_path], config[:salt_file_root]) %>
   - <%= os_join(config[:root_path], config[:salt_spm_root], 'salt') %>

pillar_roots:
 <%= config[:pillar_env] || config[:salt_env] %>:
   - <%= os_join(config[:root_path], config[:salt_pillar_root]) %>
   - <%= os_join(config[:root_path], config[:salt_spm_root], 'pillar') %>

<% if windows_os? %>
file_client: local
winrepo_dir: <%= os_join(config[:root_path], config[:salt_file_root], 'win', 'repo') %>
winrepo_dir_ng: <%= os_join(config[:root_path], config[:salt_file_root], 'win', 'repo-ng') %>
<% end %>
